class GeneralFunctionsQueryBuilder {
  constructor(elements) {
    this.elements = elements;
  }

  toggleClass(elements, className, action){
    elements.forEach(element => {
      document.querySelector('[data-toggle-area="' + element + '"]').classList[action](className);
    });
  }

  queryChipFactory(dataType, prettyQueryText, queryText) {
    this.elements.counter++;
    this.elements.queryInputFieldInstance.addChip({
      tag: prettyQueryText
    });

    let queryChipElementIndex = this.elements.queryInputFieldInstance.$chips.length - 1;
    let queryChipElement = this.elements.queryInputFieldInstance.$chips[queryChipElementIndex];
    queryChipElement.classList.add('query-component');
    queryChipElement.setAttribute('data-type', dataType);
    queryChipElement.setAttribute('data-query', queryText);
    queryChipElement.setAttribute('draggable', 'true');

    queryChipElement.addEventListener('click', () => {this.deleteAttr(queryChipElement);});
    queryChipElement.addEventListener('dragstart', (event) => {
      // selects all nodes without target class
      let queryChips = this.elements.queryInputField.querySelectorAll('.query-component');

      // Adds a target chip in front of all draggable childnodes
      setTimeout(() => {
        let targetChipElement = Utils.HTMLToElement('<span class="chip drop-target">Drop here</span>');
        for (let element of queryChips) {
          if (element === queryChipElement.nextSibling) {continue;}
          let targetChipClone = targetChipElement.cloneNode(true);
          if (element === queryChipElement) {
            // If the dragged element is not at the very end, a target chip is also inserted at the end
            if (queryChips[queryChips.length - 1] !== element) {
              queryChips[queryChips.length - 1].insertAdjacentElement('afterend', targetChipClone);
            }
          } else {
            element.insertAdjacentElement('beforebegin', targetChipClone);
          }
          targetChipClone.addEventListener('dragover', (event) => {
            event.preventDefault();
          });
          targetChipClone.addEventListener('dragenter', (event) => {
            event.preventDefault();
            event.target.style.borderStyle = 'solid dotted';
          });
          targetChipClone.addEventListener('dragleave', (event) => {
            event.preventDefault();
            event.target.style.borderStyle = 'hidden';
          });
          targetChipClone.addEventListener('drop', (event) => {
            let dropzone = event.target;
            dropzone.parentElement.replaceChild(queryChipElement, dropzone);
            this.queryPreviewBuilder();
          });
        }
      }, 0);
    });

    queryChipElement.addEventListener('dragend', (event) => {
      let targets = document.querySelectorAll('.drop-target');
      for (let target of targets) {
        target.remove();
      }
    });

    // Ensures that metadata is always at the end of the query:
    // const lastChild = this.elements.queryInputFieldInstance.lastChild;
    // const isLastChildTextAnnotation = lastChild && lastChild.dataset.type === 'text-annotation';
    // console.log(isLastChildTextAnnotation);
    // console.log(lastChild);

    // if (!isLastChildTextAnnotation) {
    //   this.elements.queryInputFieldInstance.appendChild(queryChipElement);
    // } else {
    //   this.elements.queryInputFieldInstance.insertBefore(queryChipElement, lastChild);
    // }

    this.queryPreviewBuilder();

    // Shows a hint about possible functions for editing the query at the first added element in the query
    // if (this.elements.queryInputFieldInstance.childNodes.length === 1) {
    //   app.flash('You can edit your query by deleting individual elements or moving them via drag and drop.');
    // }
  }

  queryPreviewBuilder() {
    let queryPreview = document.querySelector('#query-preview');
    let queryChipElements = Array.from(Object.values(this.elements.queryInputFieldInstance.$chips));
    queryChipElements.pop();
    this.elements.queryInputFieldContent = [];
    queryChipElements.forEach(element => {

      let queryElement = element.dataset.query;
      if (queryElement !== undefined) {
        queryElement = Utils.escape(queryElement);
        this.elements.queryInputFieldContent.push(queryElement);
      }
    });
  
    let queryString = this.elements.queryInputFieldContent.join(' ');
    queryString += ';';
  
    queryPreview.innerHTML = queryString;
    queryPreview.parentNode.classList.toggle('hide', queryString === ';');
  }

  deleteAttr(attr) {
    if (attr.dataset.type === "start-sentence") {
      this.elements.sentenceElement.innerHTML = 'Sentence';
    } else if (attr.dataset.type === "start-entity" || attr.dataset.type === "start-empty-entity") {
      this.elements.entityElement.innerHTML = 'Entity';
    }
    let queryChipElements = Array.from(Object.values(this.elements.queryInputFieldInstance.$chips));
    queryChipElements.pop();
    this.elements.counter -= 1;
    this.elements.queryInputFieldInstance.deleteChip(queryChipElements.indexOf(attr));
    this.queryPreviewBuilder();
  }
}

